#include <stdafx.h>
#include <windows.h>
#include <conio.h>
#include <cstring>
#include <cstdlib>
#include <iostream>

#include "Serial.h" // open source  CSerial

//////////////////////////////////////////////////////
//  , 
//  -915, 916.
//////////////////////////////////////////////////////
enum InterfaceProtocol
{
  ASCII,
  BCD
};

//////////////////////////////////////////////////////
//    
// -915, 916  .
//////////////////////////////////////////////////////
enum InterfaceBaudRate
{
  BR19200,
  BR28800,
  BR38400,
  BR57600,
  BR76800,
  BR115200,
  BR230400,
  BR500000
};

const DWORD SETUP_COMMAND_LENGTH  = 7;    //   
const DWORD SETUP_ANSWER_LENGTH   = 6;    //    
const DWORD GUARANTEE_TIMEOUT     = 50;   //     

const DWORD ASCII_ANSWER_MAX_LENGTH = 13; //     ASCII 
const DWORD ASCII_COMMAND_LENGTH    = 3;  //    ASCII 

const UCHAR ASCII_START_TOKEN = 0x3e;     //   
const UCHAR ASCII_STOP_TOKEN  = 0x0d;     //   

const DWORD NO_POINT = (1 << 31) - 1;     //      (max int)

//////////////////////////////////////////////////////
//    -915, 916.
//
//  <command>     -   ,
//  <address>     -  ,
//  <capacity>    -  ,
//  <protocol>    -  ,
//  <baud>        -  ,
//  <useAlarmBit> -    ALARM.
//////////////////////////////////////////////////////
void MakeSetupCommand(UCHAR* command,
            UCHAR address,
            UCHAR capacity = 0,
            InterfaceProtocol protocol = ASCII, 
            InterfaceBaudRate baud = BR19200, 
            bool useAlarmBit = false)
{
  command[0] = 35;    // #
  command[1] = 112;   // p  
  command[2] = 35;    // #

  command[3] = address; 
  command[4] = protocol;
  command[5] = baud;
  
  if (useAlarmBit) ++capacity;

  command[6] = capacity;
}

//////////////////////////////////////////////////////
//   -915, 916.
//
//  <portName>    -   (COM), 
//                   
//                 ,
//  <address>     -  ,
//  <capacity>    -  ,
//  <protocol>    -  ,
//  <baud>        -  ,
//  <useAlarmBit> -    ALARM.
//////////////////////////////////////////////////////
bool SetupInterface(const LPCTSTR portName, 
          UCHAR address,
          UCHAR capacity = 0,
          InterfaceProtocol protocol = ASCII, 
          InterfaceBaudRate baud = BR19200, 
          bool useAlarmBit = false)
{
  bool success = false;

  if (CSerial::CheckPort(portName) == CSerial::EPortAvailable)
  {
    //
    //    .
    //

    UCHAR command[SETUP_COMMAND_LENGTH];
    MakeSetupCommand(command, address, capacity, protocol, baud, useAlarmBit);

    //
    //    .
    //

    CSerial port;
    
    if (port.Open(portName) == NO_ERROR &&
      port.Setup(CSerial::EBaud19200) == NO_ERROR)
    {
      port.Purge();

      if (port.Write(command, SETUP_COMMAND_LENGTH) == NO_ERROR)
      {
        //
        //  ,    
        // ,     .
        //

        Sleep(GUARANTEE_TIMEOUT);

        //
        //    
        //   .
        //

        UCHAR answer[SETUP_ANSWER_LENGTH];
        if (port.Read(answer, SETUP_ANSWER_LENGTH) == NO_ERROR)
        {
          //
          //     
          // [1; 4]      
          // [3; 6]   .
          //
          
          success = true;
          for (int i = 1, stop = SETUP_ANSWER_LENGTH - 1; i < stop; ++i)
          {
            if (command[i + 2] != answer[i])
            {
              success = false;
              break;
            }
          }
        }
      }
      
      port.Close();
    }
  }

  return success;
}

//////////////////////////////////////////////////////
//    .
//
//  <name>      -  ,
//  <coord>     -  .
//////////////////////////////////////////////////////
void PrintCoordinate(char *name, int coord)
{
  std::cout << name;

  if (coord != NO_POINT) 
    std::cout << coord;
  else 
    std::cout << "";

  std::cout << std::endl;
}

//////////////////////////////////////////////////////
//    ASCII .
//
//  <buf>       - ASCII ,
//  <return>    -  .
//////////////////////////////////////////////////////
int GetCoordinate(const UCHAR *buf)
{
  int coord = NO_POINT;
  if (buf[0] == ASCII_START_TOKEN)
  {
    //
    //     .
    //

    int length = 0;
    for (; length < ASCII_ANSWER_MAX_LENGTH; ++length)
    {
      if (buf[length] == ASCII_STOP_TOKEN)
      {
        break;
      }
    }

    if (length != 0)
    {
      //
      //      
      //   1 (  '\0').
      //
      char *chc = new char[length];

      memcpy(chc, (buf + 1), length - 1);
      chc[length - 1] = '\0';

      coord = atoi(chc);

      delete[] chc;
    }
  }

  return coord;
}

int main()
{
  //    .
  setlocale(LC_ALL, "russian");

  //
  //     
  //   .
  //

  const LPCTSTR portName = "COM1";
  if (CSerial::CheckPort(portName) != CSerial::EPortAvailable)
  {
    std::cerr << "    " << portName << "." << std::endl;
    return 1;
  }
  
  //    .
  UCHAR address = 1;

  //
  //      ASCII , 
  //   1,  0,   19200 /.
  //

  char program = 'n';
  std::cout << " ? (y/n): "; std::cin >> program; 
  std::cout << std::endl;

  if (program == 'y')
  {
    std::cout << "    \"\","                   << std::endl 
              << ",   ."                       << std::endl
              << "     ."  << std::endl
              << "     ..."  << std::endl;

    while (_getch() == 0)
    {
      Sleep(50);
    }

    if (!SetupInterface(portName, address, 0, ASCII, BR115200))
    {
      std::cerr << "   ." << std::endl;
      return 2;
    }
    else
    {
      std::cout << " ." << std::endl << std::endl;
    }
  }

  //
  //    
  // COM-   .
  //

  CSerial port;

  if (port.Open(portName) != NO_ERROR)
  {
    std::cerr << "      ." << std::endl;
    return 3;
  }
  
  port.Setup(CSerial::EBaud115200, CSerial::EData8, CSerial::EParNone, CSerial::EStop1);  
 

  port.Purge();

  UCHAR relCmd[] = { 35, address, 111 };  //   
  UCHAR refCmd[] = { 35, address, 114 };  //   . 
  UCHAR absCmd[] = { 35, address, 97  };  //   

  UCHAR answer[ASCII_ANSWER_MAX_LENGTH];

  //
  //  .
  //

  int coord = NO_POINT;

  char resume = 'y';
  while (resume != 'n')
  {
    //
    //   , 
    //      .
    //
    //  : 
    //    -  , 
    //    -     , 
    //    -  ,
    //    -  .
    //
    
    if (port.Write(relCmd, ASCII_COMMAND_LENGTH) == NO_ERROR)
    {
      Sleep(GUARANTEE_TIMEOUT);

      if (port.Read(answer, ASCII_ANSWER_MAX_LENGTH) == NO_ERROR)
      {
        coord = GetCoordinate(answer);
      }
    }
    PrintCoordinate(" : ", coord);

    if (port.Write(absCmd, ASCII_COMMAND_LENGTH) == NO_ERROR)
    {
      Sleep(GUARANTEE_TIMEOUT);

      if (port.Read(answer, ASCII_ANSWER_MAX_LENGTH) == NO_ERROR)
      {
        coord = GetCoordinate(answer);
      }
    }
    PrintCoordinate(" : ", coord);

    if (port.Write(refCmd, ASCII_COMMAND_LENGTH) == NO_ERROR)
    {
      Sleep(GUARANTEE_TIMEOUT);

      if (port.Read(answer, ASCII_ANSWER_MAX_LENGTH) == NO_ERROR)
      {
        coord = GetCoordinate(answer);
      }
    }
    PrintCoordinate("  : ", coord);

    std::cout << std::endl << " ? (y/n): "; std::cin >> resume;
    std::cout << std::endl;
  }

  //  .
  port.Close();

  return 0;
}
